/* COPYRIGHT NOTICE
 *
 * This program comes from "The X Toolkit Cookbook" by Paul E. Kimball
 * c. 1995 Prentice Hall.
 *
 *  HDR1: Example 7-1 - children.c - Geometry management
 *  
 *  HDR2: Chapter 7, page 144
 *
 * This software is provided free of charge and may be freely distributed
 * in its original form, so long as this notice appears unchanged in each 
 * copy.  This software is supplied "as-is" and is not covered by warrantee
 * of any kind, express or implied.  
 *
 * This and other examples are available by anonymous ftp from
 * ftp.netcom.com, directory "pub/pk/pkimball/cookbook".
 *
 * 1/15/95	Paul Kimball - pkimball@netcom.com
 */

/***  Include required header files       ****************************/

#include <X11/Intrinsic.h>    /* The Intrinsics                      */
#include <Xm/Xm.h>            /* Motif symbols and data types        */
#include <Xm/PushB.h>         /* Motif XmPushButton                  */
#include <Xm/RowColumn.h>     /* Motif XmRowColumn                   */
#include <Xm/BulletinB.h>     /* Motif XmBulletinBoard               */
#include <Xm/Form.h>          /* Motif XmForm                        */
#include <Xm/ScrolledW.h>     /* Motif XmScrolledWindow              */
#include <Xm/PanedW.h>        /* Motif XmPanedWindow                 */

/***  Declare global variables      **********************************/

XtAppContext  context;             /* application context            */

Widget  toplevel, manager;         /* widget tree                    */
Widget  child_1, child_2, child_3, child_4, child_5;

Dimension init_width, init_height; /* child_1 initial size           */

/***  type of manager  - uncomment only one    ***********************/

#define MANAGER_CLASS xmRowColumnWidgetClass

/*#define MANAGER_CLASS xmBulletinBoardWidgetClass  */
/*#define MANAGER_CLASS xmFormWidgetClass           */
/*#define MANAGER_CLASS xmPanedWindowWidgetClass    */
/*#define MANAGER_CLASS xmScrolledWindowWidgetClass */

/***  Forward declare callback routines    ***************************/

void Grow();                         /* Make calling widget larger   */
void Unmanage();                     /* Unmanage calling widget      */
void Unmap();                        /* Unmap calling widget         */
void Reset();                        /* Reset buttons to normal      */
void ExitCallback();                 /* Exit                         */
void InitInterface();                /* Initialize user interface    */

/***  Main program  **************************************************/

main(argc,argv)
int  argc;
char **argv;
{

/***  Init. Toolkit, App. Context, Application Shell    **************/

 toplevel = XtAppInitialize( &context, "Children", 
     NULL, 0, &argc, argv, NULL, NULL, 0 );

/***  Create widget tree, bind callbacks  ****************************/

 manager = XtCreateManagedWidget( "manager",    /* Create manager    */
     MANAGER_CLASS, toplevel, NULL, 0);

 child_1 = XtCreateManagedWidget( "child_1",    /* First pushbutton  */
     xmPushButtonWidgetClass, manager, NULL, 0);
 XtAddCallback( child_1,                        /* Grow callback     */
     XmNactivateCallback, Grow , NULL );

 child_2 = XtCreateManagedWidget( "child_2",    /* Second pushbutton */
     xmPushButtonWidgetClass, manager, NULL, 0);
 XtAddCallback( child_2,                        /* Unmanage callback */
     XmNactivateCallback, Unmanage, NULL );

 child_3 = XtCreateManagedWidget("child_3",     /* Third pushbutton  */
     xmPushButtonWidgetClass, manager, NULL, 0);
 XtAddCallback( child_3,                        /* Unmap callback    */
     XmNactivateCallback, Unmap, NULL );

 child_4 = XtCreateManagedWidget("child_4",     /* Fourth pushbutton */
     xmPushButtonWidgetClass, manager, NULL, 0);
 XtAddCallback( child_4,                        /* Reset callback    */
     XmNactivateCallback, Reset, NULL );

 child_5 = XtCreateManagedWidget("child_5",     /* Fifth pushbutton  */
     xmPushButtonWidgetClass, manager, NULL, 0);
 XtAddCallback( child_5,                        /* Exit callback     */
     XmNactivateCallback, ExitCallback, NULL );

 InitInterface();       /* Set resources unsettable in resource file */

 XtRealizeWidget(toplevel);                     /* Realize everyone  */
 XtAppMainLoop(context);                        /* Start main loop   */

}

/***  Application Callback Routines    *******************************/

void Grow(w, client_data, call_data)        /* Grow calling widget   */
Widget w; 
XtPointer client_data; 
XtPointer call_data;
{
  Dimension width, height; 
  XtVaGetValues(w, XmNwidth, &width,        /* Get current size      */
                   XmNheight, &height, NULL);
  XtVaSetValues(w, XmNwidth, width+10,      /* Grow by 10 pixels     */
                   XmNheight, height+10, NULL);
}
void Unmanage(w, client_data, call_data)    /* Unmanage a widget     */
Widget w; 
XtPointer client_data; 
XtPointer call_data;
{
  XtUnmanageChild(w);                       /* Unmanage caller       */
}

/*********************************************************************/

void Unmap(w, client_data, call_data)       /* Unmap a widget        */
Widget w; 
XtPointer client_data; 
XtPointer call_data;
{
  XtSetMappedWhenManaged( w, False );
  XtUnmapWidget(w);                         /* Unmap calling widget  */
}

/*********************************************************************/

void Reset(w, client_data, call_data)       /* reset user interface  */
Widget w; 
XtPointer client_data; 
XtPointer call_data;
{
  XtVaSetValues(child_1,                    /* Restore child 1 size  */
     XmNwidth, init_width, 
     XmNheight, init_height, NULL);
  XtManageChild(child_2);                   /* Manage child 2 again  */
  XtMapWidget(child_3);                     /* Map child 3 again     */
}

/*********************************************************************/

void InitInterface()        /* save initial state of user interface  */
{                           /* and set resources that cannot be set  */
                            /* in resource files                     */
  XtVaGetValues(child_1,
     XmNwidth, &init_width, XmNheight, &init_height, NULL);
}

/*********************************************************************/

void ExitCallback(w, client_data, call_data)
Widget w; 
XtPointer client_data; 
XtPointer call_data;
{
  exit();
}
