/* COPYRIGHT NOTICE
 *
 * This program comes from "The X Toolkit Cookbook" by Paul E. Kimball
 * c. 1995 Prentice Hall.
 *
 *  HDR1: Example 11-3 - gafocus.c - Using a globally active focus model
 *  
 *  HDR2: Chapter 11, page 345
 *
 * This software is provided free of charge and may be freely distributed
 * in its original form, so long as this notice appears unchanged in each 
 * copy.  This software is supplied "as-is" and is not covered by warrantee
 * of any kind, express or implied.  
 *
 * This and other examples are available by anonymous ftp from
 * ftp.netcom.com, directory "pub/pk/pkimball/cookbook".
 *
 *  1/15/95	Paul Kimball - pkimball@netcom.com
 */

/*****  Include Required Header Files    *****************************/

#include <X11/Intrinsic.h>     /* The Intrinsics                     */
#include <Xm/MainW.h>          /* Motif XmMainWindow                 */
#include <Xm/PushB.h>          /* Motif XmPushButton                 */

/*****  Declare Global Variables   ***********************************/

void HandleProtocols();        /* Ev. Handler - handle WM protocols  */
void SwitchFocus();            /* Set focus to other window          */
int FocusErrorHandler();       /* Trap protocol error on set focus   */

Atom wmprot[2];                /* WM_TAKE_FOCUS and WM_DELETE_WINDOW */
Atom _XA_WM_PROTOCOLS;         /* WM_PROTOCOLS atom                  */

/*****  Start of main()   ********************************************/

main(argc, argv)
int argc; 
char **argv;
{

 XtAppContext context;                        /* Application context */

 Widget toplevel, main1, pb1,                 /* Widget tree         */
        secondary, main2, pb2; 

/*****  Init. Toolkit, Appl. Context, Application Shell  *************/

 toplevel = XtVaAppInitialize(&context, "MyApp", NULL, 0, &argc, argv,
     NULL, XtNinput, False, NULL );

/*****  Create widget tree, bind callbacks  **************************/

 main1 = XtVaCreateManagedWidget("main1", xmMainWindowWidgetClass,
     toplevel, NULL );

 /* Create window 1 */
 pb1 = XtVaCreateManagedWidget("pb1", xmPushButtonWidgetClass, 
     main1, NULL );
 XtAddCallback(pb1, XmNactivateCallback, SwitchFocus, &pb2);
 XtRealizeWidget(toplevel);

 /* Create window 2 */
 secondary = XtVaCreatePopupShell("secondary",
     topLevelShellWidgetClass, toplevel, XtNinput, False, NULL);
 main2 = XtVaCreateManagedWidget("main2", xmMainWindowWidgetClass,
     secondary, NULL );
 pb2 = XtVaCreateManagedWidget("pb2", xmPushButtonWidgetClass, 
     main2, NULL );
 XtAddCallback(pb2, XmNactivateCallback, SwitchFocus, &pb1);
 XtRealizeWidget(secondary);
 XtPopup( secondary, XtGrabNone );

/** Establish active input model by preparing to handle WM_PROTOCOLS */

 wmprot[0] = XInternAtom(XtDisplay(toplevel), 
     "WM_DELETE_WINDOW", False);
 wmprot[1] = XInternAtom(XtDisplay(toplevel), 
     "WM_TAKE_FOCUS", False);
 _XA_WM_PROTOCOLS = XInternAtom(XtDisplay(toplevel), 
     "WM_PROTOCOLS", False);

 XSetWMProtocols(XtDisplay(toplevel), XtWindow(toplevel), wmprot,
     XtNumber(wmprot));
 XSetWMProtocols(XtDisplay(secondary), XtWindow(secondary), wmprot,
     XtNumber(wmprot));

 XtAddEventHandler(toplevel, NoEventMask, True,    /* Both windows   */
     HandleProtocols, NULL);                       /* should handle  */
 XtAddEventHandler(secondary, NoEventMask, True,   /* protocol msgs  */
     HandleProtocols, NULL);

 XtAppMainLoop(context);
}

/*********************************************************************/

void HandleProtocols( w, client, event, dispatch)
Widget w;                                    /* Shell that got event */
XtPointer client;                            /* Unused               */
XEvent *event;                               /* Event to be handled  */
Boolean *dispatch;                           /* Unused               */
{
  /** If this is WM_TAKE_FOCUS protocol then accept focus **/
  if ( (event->type == ClientMessage) 
     && (event->xclient.message_type == _XA_WM_PROTOCOLS)){ 

     /** If iconified, we should do nothing. **/
     /** handle WM_DELETE_WINDOW by exiting **/
     if ((Atom)event->xclient.data.l[0] == wmprot[0])
       exit(0);

     /** Handle WM_TAKE_FOCUS by accepting focus **/
     else if ((Atom)event->xclient.data.l[0] == wmprot[1])
       XSetInputFocus(XtDisplay(w), XtWindow(w), 
       RevertToParent, CurrentTime);
  }
}

/*********************************************************************/

void SwitchFocus( w, client, call)
Widget w;                     /* Widget that just got activated      */
XtPointer client;             /* Widget to receive focus             */
XtPointer call;
{
  Widget dest, top;           /* destination widget and its shell    */
  XErrorHandler oldhand;      /* Will hold previous error handler    */

  dest = *((Widget*)client);  /* Client data points to destination   */
  top = dest;                 /* Find nearest shell ancestor         */

  while (!XtIsSubclass(top, wmShellWidgetClass))top = XtParent(top);

  /** Set more forgiving handler in case set focus fails **/
  oldhand = XSetErrorHandler(FocusErrorHandler);

  XSetInputFocus(XtDisplay(top), XtWindow(top), 
  RevertToParent, CurrentTime);
  XSync(XtDisplay(top), False);

  /** Restore old handler and set keyboard focus to destination **/
  XSetErrorHandler(oldhand);

  XmProcessTraversal(dest, XmTRAVERSE_CURRENT);
}

/*********************************************************************/

int FocusErrorHandler(display, event)
Display *display;
XErrorEvent *event;
{
 /** Apparently XSetInputFocus got an error if we're here **/
 printf("Focus error - is your window iconified?\n");
}

