/* COPYRIGHT NOTICE
 *
 * This program comes from "The X Toolkit Cookbook" by Paul E. Kimball
 * c. 1995 Prentice Hall.
 *
 *  HDR1: Example 4-1 - hello.c - Basic toolkit programming concepts
 *  
 *  HDR2: Chapter 4, page 51
 *
 * This software is provided free of charge and may be freely distributed
 * in its original form, so long as this notice appears unchanged in each 
 * copy.  This software is supplied "as-is" and is not covered by warrantee
 * of any kind, express or implied.  
 *
 * This and other examples are available by anonymous ftp from
 * ftp.netcom.com, directory "pub/pk/pkimball/cookbook".
 *
 *  1/15/95	Paul Kimball - pkimball@netcom.com
 */

 
/********************************************
 *  hello.c - simple Motif toolkit application 
 */

/*****  Include Required Header Files    *****************************/

#include <X11/Intrinsic.h>     /* The Intrinsics                     */
#include <Xm/Xm.h>             /* Resource symbols                   */
#include <Xm/BulletinB.h>      /* Motif XmBulletinBoard widget       */
#include <Xm/PushB.h>          /* Motif XmPushButton widget          */

/*****  Declare Global Variables   ***********************************/

Display      *display;         /* Server connection                  */
XtAppContext context;          /* Application context                */
Widget       toplevel,         /* Root of widget tree                */
             manager,          /* Manager to hold button(s)          */
             exit_button;      /* A single button child              */

/*****  Forward declare callback routines   **************************/

void ExitCallback();           /* A routine that exits, in this case */

/*****  Start of main()   **********************************************/

main(argc,argv)
int  argc;                     /* argc and argv are the command line   */
char **argv;                   /* We will pass them to XtOpenDisplay   */
{

/** Init. toolkit, application context and application shell  ***********/

 XtToolkitInitialize();                   /* One-time initialization    */
 context = XtCreateApplicationContext();  /* One application context    */
 display = XtOpenDisplay(
     context,                     /* Context we just created            */
     "",                          /* Display connection (use default)   */
     "hello",                     /* Application name                   */
     "Hello",                     /* Application class                  */
     NULL, 0,                     /* options, num_options unused for now*/
     &argc,                       /* Command line - number of args      */
     argv);                       /* Command line - list of arguments   */

 toplevel = XtAppCreateShell(     /* Application Shell to root tree     */
     "hello",                     /* Application name                   */
     "Hello",                     /* Application class                  */
     applicationShellWidgetClass, /* Type of shell to create            */
     display,                     /* Display to put it on               */
     NULL, 0);                    /* args, num_args unused for now      */

/*****  Create widget tree, bind callbacks  *****************************/

 manager = XtCreateWidget(        /* Create a manager parent            */
     "manager",                   /* Name for this new widget           */
     xmBulletinBoardWidgetClass,  /* Type of widget to create           */
     toplevel,                    /* Widget ID of its parent            */
     NULL, 0);                    /* args, num_args unused              */

 exit_button = XtCreateWidget(    /* Create a primitive child           */
     "exitButton",                /* Name for this new widget           */
     xmPushButtonWidgetClass,     /* Type of widget to create           */
     manager,                     /* Widget ID of its parent            */
     NULL, 0);                    /* args, num_args unused              */

 XtAddCallback(                   /* Attach a callback to a widget      */
     exit_button,                 /* Widget which will call back        */
     XmNactivateCallback,         /* Why it will call back              */
     ExitCallback,                /* Routine to call                    */
     NULL );                      /* Client_data - unused for now       */

/*****  Manage, realize initial widget tree  ****************************/

 XtManageChild(exit_button);      /* BulletinBoard manages button       */
 XtManageChild(manager);          /* Shell manages BulletinBoard        */
 XtRealizeWidget(toplevel);       /* Force widgets to get windows       */

/*****  Start toolkit event loop  ***************************************/

 XtAppMainLoop(context);          /* Can't get past here !!!            */
}

/****** Callback Routines ***********************************************/

void ExitCallback(w, client_data, call_data)
Widget   w;                   /* Input argument - widget that called */
XtPointer client_data;        /* Input argument - application data   */
XtPointer call_data;          /* Input argument - call-specific data */
{
    exit();                   /* Doesn't do much - only exits        */
}

