/* COPYRIGHT NOTICE
 *
 * This program comes from "The X Toolkit Cookbook" by Paul E. Kimball
 * c. 1995 Prentice Hall.
 *
 *  HDR1: Example 8-8 - timer.c - Timer callbacks
 *  
 *  HDR2: Chapter 8, page 202
 *
 * This software is provided free of charge and may be freely distributed
 * in its original form, so long as this notice appears unchanged in each 
 * copy.  This software is supplied "as-is" and is not covered by warrantee
 * of any kind, express or implied.  
 *
 * This and other examples are available by anonymous ftp from
 * ftp.netcom.com, directory "pub/pk/pkimball/cookbook".
 *
 *  1/15/95	Paul Kimball - pkimball@netcom.com
 */

/***  Include Required Header Files    ***********************************/

#include <X11/Intrinsic.h>         /* The Intrinsics                     */
#include <Xm/Xm.h>                 /* Motif Symbols                      */
#include <Xm/RowColumn.h>          /* Motif XmRowColumn manager          */
#include <Xm/PushB.h>              /* Motif XmPushButton                 */
#include <Xm/Label.h>              /* Motif XmLabel                      */
#include <Xm/Text.h>               /* Motif XmText                       */

/***** Declare Global Variables  *****************************************/

Display     *display;                           /* Server connection     */
XtAppContext  context;

Widget  toplevel, manager, top_button;
Widget  bottom_button, top_label, text_area;

int          time_left;                         /* Seconds left to count */
Boolean      alarm_done;                        /* True if alarm finished*/
XtIntervalId cyclic_timer, limit_timer;         /* Timers                */

/***** Forward Declare Callback Routines *********************************/

void ExitCallback();        /* exits                                     */
void StartTimer();          /* Callback to starts parade of timeouts     */
void Countdown();           /* Timer proc that executes every second     */
void StopTimer();           /* Callback to let user stop timers          */
void Alarm();               /* Annoying timer proc rings bell repeatedly */ 
void StopAlarm();           /* timer proc that finally stops bell        */
void ClearText();           /* Cears text field when user assigns focus  */

/***** Main Program  *****************************************************/

main(argc,argv)
int argc;
char **argv;
{
 toplevel = XtAppInitialize( &context, "Timer",
     NULL, 0, &argc, argv, NULL, NULL, 0 );
     
 display = XtDisplay(toplevel); 

 manager = XtCreateManagedWidget("manager",
     xmRowColumnWidgetClass, toplevel, NULL, 0);

 top_label = XtCreateManagedWidget("topLabel",
     xmLabelWidgetClass, manager, NULL, 0);

 text_area = XtCreateManagedWidget("textArea",
     xmTextWidgetClass, manager, NULL, 0);
 XtAddCallback( text_area, XmNactivateCallback,
     StartTimer, NULL );
 XtAddCallback( text_area, XmNfocusCallback,
     ClearText, NULL ); 

 top_button = XtCreateManagedWidget("topButton",
     xmPushButtonWidgetClass, manager, NULL, 0);
 XtAddCallback( top_button, XmNactivateCallback,
     StopTimer, NULL );

 bottom_button = XtCreateManagedWidget("bottomButton",
     xmPushButtonWidgetClass, manager, NULL, 0);
 XtAddCallback( bottom_button, XmNactivateCallback,
     ExitCallback, NULL );

 XtRealizeWidget(toplevel);                    /* Realize all        */
 XtAppMainLoop(context);                       /* Start main loop    */
}

/*****  Callback Routines ********************************************/

void StartTimer(w, client_data, call_data)     /* Starts timer proc  */
Widget  w;                                     /* ID of Text widget  */
XtPointer client_data, call_data;              /* Unused             */
{
 String buffer;                                /* input string       */

 XtVaGetValues(w, XmNvalue, &buffer, NULL);    /* Get input string   */

 /* Try to scan input  */

 if( !sscanf(buffer, "%i", &time_left) ){   
   /* Bad input       */

   XtVaSetValues(w, XmNvalue, "Try again", NULL);

 }
 else {                

   /* Got valid input    */
   /* Counting; tell user and prevent new input*/

   XtVaSetValues(text_area, 
       XmNvalue, "Starting Countdown",
       XmNeditable, False, NULL );

   cyclic_timer = XtAppAddTimeOut(context, 1000, Countdown, NULL);
   
   alarm_done = False;                         /* Hasn't gone off    */
 }
}

/*********************************************************************/

void Countdown(client_data, timerid)        /* Called every second   */
XtPointer   client_data;                    /* Unused                */
XtIntervalId *timerid;                      /* this timer's ID       */
{

 char buffer[40];                           /* text message buffer   */
 time_left--;                               /* decrement seconds     */

 if (time_left == 0) {                      /* time's up! Tell user  */ 
   XBell(display, 50);
   XtVaSetValues(text_area, XmNvalue, "TIME'S UP!!!", NULL);

   /* start new cyclic timer to ring alarm, limit timer to stop it   */  
   cyclic_timer = XtAppAddTimeOut(context, 500, Alarm, NULL);
   limit_timer = XtAppAddTimeOut(context, 5000, StopAlarm, NULL);

 }
 else {

   /* Still counting - update display and set timer to call again    */
   sprintf(buffer, "%i seconds left", time_left);
   XtVaSetValues(text_area, XmNvalue, buffer, NULL); 

   cyclic_timer = XtAppAddTimeOut(context, 1000, Countdown, NULL);
 }
}

/*********************************************************************/

void StopTimer(w, client_data, call_data)   /* Cancels the timer     */
Widget  w;                                  /* Unused                */
XtPointer client_data, call_data;           /* Unused                */
{

 if(cyclic_timer != NULL)                   /* be sure timer exists      */
    XtRemoveTimeOut(cyclic_timer);          /* stop cyclic timer if any  */

 if(limit_timer != NULL)                    /* be sure timer exists      */
    XtRemoveTimeOut(limit_timer);           /* stop limit timer if any   */

 /* Count was reset - tell user and allow new input       */
 XtVaSetValues(text_area, 
     XmNvalue, "Reset", 
     XmNeditable, True, NULL );
}

/*********************************************************************/

void Alarm(client_data, timerid)        /* Cyclic timer to ring bell */
XtPointer client_data;                  /* Unused                    */
XtIntervalId *timerid;                  /* Unused                    */
{
 if (alarm_done) {        
   /* alarm timed out */
   cyclic_timer = NULL;                 /* set flag to avoid error   */
   XtVaSetValues(text_area,             /* Allow new input           */
      XmNeditable, True, 
      NULL );
   return;                              /* Don't requeue timer       */
 }
 else {              
   /* Alarm has time left       */
   /* Ring annoying bell and Requeue to ring again     */   
   XBell(display, 50);
   cyclic_timer = XtAppAddTimeOut(context, 500, Alarm, NULL);
 }
}

/*********************************************************************/

void StopAlarm(client_data, timerid)        /* Alarm limit timer     */
XtPointer client_data;                      /* unused                */
XtIntervalId *timerid;
{
 alarm_done = True;                    /* alarm timed out - set flag */
 limit_timer = NULL;                   /* set flag to avoid error    */
}

/*********************************************************************/

void ClearText(w, client_data, call_data)      /* Clear text field   */
Widget  w;                                     /* Text widget        */
XtPointer client_data, call_data;              /* Unused             */
{
   XtVaSetValues(w, XmNvalue, "", NULL);
}

/*********************************************************************/

void ExitCallback(w, client_data, call_data)   /* Exits              */
Widget  w;                                     /* Unused             */
XtPointer client_data, call_data;              /* Unused             */
{
   exit(1);
}
